/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
    #include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
    #include <htc.h>        /* HiTech General Include File */
#endif
#include <stdint.h>         /* For uint8_t definition */

#include "lcd.h"
#include <string.h>

/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/

uint8_t BLACKLIGHT = 0;

void LCD_Set_BlackLight(uint8_t value){
	BLACKLIGHT = value;
}
void lcd_wait(volatile unsigned long delay){
	while (delay--);
}

void LCD_SEND_CHR(unsigned char character){
	lcd_wait(110);
	PORTA = (PORTA & 0xF0) | character >> 4;
	RS_ON;
	lcd_wait(1);
	EN_ON;
	lcd_wait(2);
	EN_OFF;
	PORTA = (PORTA & 0xF0) | (character & 0x0F);
	RS_ON;
	lcd_wait(1);
	EN_ON;
	lcd_wait(2);
	EN_OFF;
}

void LCD_SEND_CMD(unsigned char command){


	lcd_wait(110); // wait Twrite = 43us; Twrite = 100 x Twait = 43.75ms
	PORTA = (PORTA & 0xF0) | command >> 4;

	RS_OFF;
	lcd_wait(1);
	EN_ON;
	lcd_wait(2);
	EN_OFF;
	PORTA = (PORTA & 0xF0) | (command & 0x0F);
	RS_OFF;
	lcd_wait(1);
	EN_ON;
	lcd_wait(2);
	EN_OFF;
}


void LCD_Send_Symbol(unsigned char row, unsigned char position, unsigned char symbol){
	if (row > 2) return;
	//Set row
	if (row == 1) LCD_SEND_CMD(0x80); // Set DDRAM address 0x00
	else LCD_SEND_CMD(0xC0); // Set DDRAM address 0x40
	//Set symbol position
	if (position > 15) return;
	else {
		while (position) {
			LCD_SEND_CMD(0x14); // Cursor or display shift (Shift cursor to the right, AC is increased by 1)
			position--;
		}
	}
	//Write symbol
	LCD_SEND_CHR(symbol);
}

void LCD_Clear(void) {
	LCD_SEND_CMD(0x01); // ClearDisplay
	lcd_wait(3510); //wiat Tclear = 1.53ms; 1.53ms = Y x 437.5ns => Y = 3500
}


void LCD_Init(void) {


	// Wait time more than 40ms  so that target voltage to be set
	lcd_wait(10000); // Twait x 100000  = 437.5ns x 100000 = 43.75ms
	// This first commands have 4 bits Length!!!!! Not Byte!!!
	// TIME>15ms;
	lcd_wait(10000); // Twait x 100000 = 437.5ns x 100000 = 43.75ms
	RS_OFF;


	PORTA = (PORTA & 0xF0) | 0x03;
	EN_ON;
	lcd_wait(2); // wiat Tw = 300ns; Tw = 2 x Twait = 875ns
	EN_OFF;

	// TIME>4,1ms;
	lcd_wait(10000); // Twait x 100000 = 437.5ns x 100000 = 43.75ms
	PORTA = (PORTA & 0xF0) | 0x03;
	EN_ON;
	lcd_wait(2); // wiat Tw = 300ns; Tw = 2 x Twait = 875ns
	EN_OFF;

	// Time>100us;
	lcd_wait(10000); // Twait x 10000 = 437.5ns x 10000 = 4.375ms
	PORTA = (PORTA & 0xF0) | 0x03;
	EN_ON;
	lcd_wait(2); // wiat Tw = 300ns; Tw = 2 x Twait = 875ns
	EN_OFF;

	// TIME>4,1ms;
	lcd_wait(10000); // Twait x 100000 = 437.5ns x 100000 = 43.75ms
	PORTA = (PORTA & 0xF0) | 0x02;
	EN_ON;
	lcd_wait(2); // wiat Tw = 300ns; Tw = 2 x Twait = 875ns
	EN_OFF;

	// Next commands have 8 bits Length i.e. one Byte !!!
	lcd_wait(1500);
	LCD_SEND_CMD(0x28); // FUNCTION SET
	lcd_wait(1500);
	LCD_SEND_CMD(0x08); // DISPLAY OFF
	lcd_wait(1500);
	LCD_SEND_CMD(0x01); // DISPLAY CLEAR
	lcd_wait(1500);
	LCD_SEND_CMD(0x06); // Entry mode set
	lcd_wait(1500);
	LCD_SEND_CMD(0x0C); // DISPLAY ON
	lcd_wait(1500);

}










